﻿using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;

namespace SoftArcs.CoreClassExtensions
{
	public static class StringClassExtensions
	{
		private const string WORD_SPLIT_PATTERN = @"((\b[^\s]+\b)((?<=\.\w).)?)";

		/// <summary>
		/// Split a string into single words with respect to punctuation marks.
		/// </summary>
		/// <param name="strToSplit">The string to split.</param>
		/// <returns>All words as an Enumerable of strings.</returns>
		public static IEnumerable<string> SplitIntoWords(this string strToSplit)
		{
			return (from Match word in Regex.Matches( strToSplit, WORD_SPLIT_PATTERN ) select word.ToString()).AsEnumerable();
		}

		/// <summary>
		/// Count the words in a string.
		/// </summary>
		/// <param name="strToCount">The string to count.</param>
		/// <returns>The count of all words.</returns>
		public static int CountWords(this string strToCount)
		{
			return Regex.Matches( strToCount, WORD_SPLIT_PATTERN ).Count;
		}
	}
}
