﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using BitlyDotNET.Implementations;
using SoftArcs.CoreClassExtensions;
using SoftArcs.Security;

namespace SoftArcs.UrlServices
{
	public static class ShorteningService
	{
		private static BitlyService bitlyService;

		/// <summary>
		/// Initialize the BitlyService with the given API credentials.
		/// </summary>
		/// <param name="apiToken">A valid API token.</param>
		public static void InitializeService(APIToken apiToken)
		{
			bitlyService = new BitlyService( apiToken.Key, apiToken.Secret );
		}

		/// <summary>
		/// Shorten all URLs in the given string.
		/// </summary>
		/// <param name="textWithUrls">A string with possibly containing URLs.</param>
		/// <returns>A Dictionary of strings where the key is the original URL and the value is the shorten URL.</returns>
		public static Dictionary<string, string> ShortenAllOccuringUrls(string textWithUrls)
		{
			var uriMappings = new Dictionary<string, string>();

			if (bitlyService == null)
			{
				Debug.WriteLine( "Initialisieren Sie den BitlyService mit der statischen Methode 'InitializeService(APIToken)' !" );
				return uriMappings;
			}

			var words = textWithUrls.SplitIntoWords();
			var possibleUris = words.Where( word => word.Contains( "." ) );

			foreach (var possibleUri in possibleUris)
			{
				try
				{
					var uriString = new UriBuilder( possibleUri ).Uri.ToString();

					if (uriString.Length <= 21) continue; // Alle URL < 22 Zeichen nicht kürzen !

					try
					{
						uriMappings.Add( possibleUri, bitlyService.Shorten( possibleUri ) );
					}
					catch (Exception exception)
					{
						Debug.WriteLine( "BitlyService Fehler : " + exception.Message );
					}
				}
				catch (Exception exception)
				{
					Debug.WriteLine( "UriBuilder Fehler : " + exception.Message );
				}
			}

			return uriMappings;
		}
	}
}
