using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Threading;

namespace SoftArcs.UrlServices.UI.AttachedProperties
{
	public class UrlAutoShortening
	{
		private const double TYPING_DELAY_TIME = 300.0;

		private static BackgroundWorker urlShortener;
		private static DispatcherTimer timer;
		private static TextBox textBox;

		public static bool GetIsActive(DependencyObject dpo)
		{
			return (bool)dpo.GetValue( IsActiveProperty );
		}
		public static void SetIsActive(DependencyObject dpo, bool value)
		{
			dpo.SetValue( IsActiveProperty, value );
		}
		public static readonly DependencyProperty IsActiveProperty =
			 DependencyProperty.RegisterAttached( "IsActive", typeof( bool ),
															  typeof( UrlAutoShortening ),
															  new FrameworkPropertyMetadata( false, IsActiveChanged ) );

		private static void IsActiveChanged(DependencyObject sender, DependencyPropertyChangedEventArgs e)
		{
			textBox = sender as TextBox;

			if ((bool)e.NewValue)
			{
				if (textBox != null) textBox.TextChanged += AssociatedObject_OnTextChanged;

				urlShortener = new BackgroundWorker();
				urlShortener.DoWork += urlShortener_DoWork;
				urlShortener.RunWorkerCompleted += urlShortener_RunWorkerCompleted;

				timer = new DispatcherTimer { Interval = TimeSpan.FromMilliseconds( TYPING_DELAY_TIME ) };
				timer.Tick += TimerOnTick;
			}
			else
			{
				timer.Stop();
				if (textBox != null) textBox.TextChanged -= AssociatedObject_OnTextChanged;
			}
		}

		private static void AssociatedObject_OnTextChanged(object sender, TextChangedEventArgs e)
		{
			var associatedTextBox = (sender as TextBox);
			if (associatedTextBox == null || urlShortener.IsBusy) return;

			timer.Stop();
			timer.Start();
		}

		private static void TimerOnTick(object sender, EventArgs e)
		{
			timer.Stop();
			urlShortener.RunWorkerAsync( textBox.Text );
		}

		private static void urlShortener_DoWork(object sender, DoWorkEventArgs e)
		{
			e.Result = ShorteningService.ShortenAllOccuringUrls( e.Argument.ToString() );
		}

		private static void urlShortener_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
		{
			var uriMappings = e.Result as Dictionary<string, string>;

			if (uriMappings != null)
				foreach (var uriMapping in uriMappings)
				{
					textBox.Text = textBox.Text.Replace( uriMapping.Key, uriMapping.Value );
					textBox.ToolTip += uriMapping.Key + " => " + uriMapping.Value + "\n";
				}

			textBox.SelectionStart = textBox.Text.Length;
		}
	}
}
