﻿using System;
using System.Globalization;
using System.Windows.Data;
using System.Windows.Markup;
using System.Windows.Media;

namespace SoftArcs.ValueConverter
{
	/// <summary>
	/// Converts a String into a Brush (dependent on the character count).
	/// </summary>
	[ValueConversion( typeof( string ), typeof( Brush ) )]
	[MarkupExtensionReturnType( typeof( StringToCharCountBrushConverter ) )]
	public class StringToCharCountBrushConverter : MarkupExtension, IValueConverter
	{
		public int MaxCount { get; set; }

		public Brush DefaultBrush { get; set; }

		public Brush MaxCountExceededBrush { get; set; }

		public StringToCharCountBrushConverter()
		{
			DefaultBrush = Brushes.Transparent;
			MaxCountExceededBrush = Brushes.Crimson;
		}

		#region IValueConverter Members

		public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
		{
			var stringToCheck = value as string;

			if (stringToCheck != null)
			{
				stringToCheck = stringToCheck.Replace( "\r", "" ).Replace( "\n", "" );

				if (MaxCount == 0 || MaxCount >= stringToCheck.Length) return DefaultBrush;

				return MaxCountExceededBrush;
			}

			return DefaultBrush;
		}

		public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
		{
			// No suitable implementation possible !
			throw new NotSupportedException();
		}

		#endregion // IValueConverter Members

		#region MarkupExtension "overrides"

		/// <summary>
		/// Overrides the abstract "ProvideValue" method from the "MarkupExtension" class
		/// </summary>
		public override object ProvideValue(IServiceProvider serviceProvider)
		{
			return new StringToCharCountBrushConverter
				{
					MaxCount = MaxCount,
					DefaultBrush = DefaultBrush,
					MaxCountExceededBrush = MaxCountExceededBrush
				};
		}

		#endregion
	}
}
