﻿using System;
using System.Globalization;
using System.Windows.Data;
using System.Windows.Markup;

namespace SoftArcs.ValueConverter
{
	/// <summary>
	/// Converts a String into an Integer value with the count of characters. When MaxCount is given, the
	/// Integer value will hold the remaining count of characters.
	/// </summary>
	[ValueConversion( typeof( string ), typeof( int ) )]
	[MarkupExtensionReturnType( typeof( StringToCharCountConverter ) )]
	public class StringToCharCountConverter : MarkupExtension, IValueConverter
	{
		public int MaxCount { get; set; }

		// ReSharper disable EmptyConstructor
		public StringToCharCountConverter() { } // ReSharper restore EmptyConstructor

		#region IValueConverter Members

		public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
		{
			var stringToCheck = value as string;

			if (stringToCheck != null)
			{
				stringToCheck = stringToCheck.Replace( "\r", "" ).Replace( "\n", "" );

				if (MaxCount == 0) return stringToCheck.Length;

				return MaxCount - stringToCheck.Length;
			}

			return MaxCount != 0 ? MaxCount : value;
		}

		public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
		{
			// No suitable implementation possible !
			throw new NotSupportedException();
		}

		#endregion // IValueConverter Members

		#region MarkupExtension "overrides"

		/// <summary>
		/// Overrides the abstract "ProvideValue" method from the "MarkupExtension" class
		/// </summary>
		public override object ProvideValue(IServiceProvider serviceProvider)
		{
			return new StringToCharCountConverter { MaxCount = MaxCount };
		}

		#endregion
	}
}
